/***************************************************************************

		160_link.asm

        Link based boot loader of the ADSP21160 processor.

        Copyright (c) 1999 Analog Devices Inc.  All rights reserved.

*****************************************************************************/

/* Warning: After booting process, EP0I bit in IMASK is set.				*/

/* Define the addresses of various IOP registers							*/

#include "def21160.h"

.SEGMENT/PM	seg_ldr;

/* The loader begins with the interrupts up to and including the low 	*/
/*  priority timer interrupt.											*/

		NOP;NOP;NOP;NOP;				/* Reserved interrupt			*/

___lib_RSTI:	IDLE;					/* Implicit IDLE instruction	*/
				JUMP Start_Loader (DB);	/* Begin loader					*/
				nop;					/* Pad to next interrupt		*/
				nop;					/* Pad to next interrupt		*/

/* Vector for illegal input condition detected:							*/
___lib_IICD: 	NOP;
				NOP;
				RTI;
				RTI;

/* Vector for status stack/loop stack overflow or PC stack full:		*/
___lib_SOVFI:   NOP;
				NOP;
                RTI;
                RTI;

/* Vector for high priority timer interrupt:							*/
___lib_TMZHI:   NOP;
				NOP;
                RTI;
                RTI;

/* Vector for Vector Interrupt:											*/
___lib_VIRPTI:  NOP;
				NOP;
                RTI;
                RTI;
                
/* Vector for Hardware Interrupt 2 (IRQ2):								*/
___lib_IRQ2I:   NOP;
				NOP;
                RTI;
                RTI;

/* Vector for Hardware Interrupt 1 (IRQ1):								*/
___lib_IRQ1I:   NOP;
				NOP;
                RTI;
                RTI;

/* Vector for Hardware Interrupt 0 (IRQ0):								*/
___lib_IRQ0I:   NOP;
				NOP;
                RTI;
                RTI;

                NOP;NOP;NOP;NOP;		/* Reserved interrupt			*/

/* Vectors for Serial port DMA channels:								*/
___lib_SPR0I:   NOP;
				NOP;
                RTI;
                RTI;

___lib_SPR1I:   NOP;
				NOP;
                RTI;
                RTI;

___lib_SPT0I:   NOP;
				NOP;
                RTI;
                RTI;

___lib_SPT1I:   NOP;
				NOP;
                RTI;
                RTI;

/* Vectors for link port DMA channels:									*/
___lib_LP0I : 	NOP;
				NOP;
				RTI;
				RTI;

___lib_LP1I : 	NOP;
				NOP;
				RTI;
				RTI;

___lib_LP2I:    NOP;
				NOP;
                RTI;
                RTI;

___lib_LP3I:    NOP;
				NOP;
                RTI;
                RTI;

___lib_LP4I:    RTI (DB);
                nop;
                bit clr lirptl 0x100000;
                RTI;

___lib_LP5I:    NOP;
				NOP;
                RTI;
                RTI;

/* Vectors for External port DMA channels:								*/
___lib_EP0I:	NOP;
				NOP;
				RTI;
				RTI;

		

Start_Loader:	
/* After power up or reset, default value in SYSCON register is 0x00000010	*/
/* and WAIT register is 0x01ce739c. If for any reason these two value should*/
/* be modified, do it here as following example. Make sure this file does  	*/
/* not exceed 256 words.                                                    */
/*                                                                          */
/*              r0= 0xXXXXXXXX;                                            	*/
/*              DM(SYSCON)= r0;      Modify SYSCON                          */
/*                                                                          */
/*              r0= 0xXXXXXXXX;                                             */
/*              DM(WAIT)= r0;        Modify WAIT                            */
/*                                                                          */
/*                                                                          */

		ustat1 = dm(SYSCON);	/* Clear Buffer Hang Disable	*/
		bit clr ustat1 BHD;		/* for rev 0.0					*/
		dm(SYSCON) = ustat1;

		L0=0;					/* Zero out L-registers so they	*/
		L4=0;					/* can be used without wrap		*/
		L8=0;
		L12=0;
		L15=0;

		M5=0;					/* Setup M-registers to use for	*/			
		M6=1;					/* for various memory transfers */
		M13=0;
		M14=1;
				
		R11=DM(SYSCON);			/* Read current SYSCON setting  */
		R12=PASS R11;			/* Hold Initial SYSCON setting  */

		I12 = SYSCON;			/* Address of SYSCON			*/

		I15 = 0x40004;

		R8 = 0x0;				/* Clear LCOM					*/
		dm(LCOM)=R8;
	
		R8 = 0x04c00;			/* Lbuf4 enabled, DMA enabled, receive,  48bits	*/
		R9 = 0x04400;			/* Lbuf4 enabled, DMA disabled, receive, 48bits */
		
		dm(LCTL1) = R9; 		/* Lbuf4 enabled, DMA disabled, receive, 48bits */

read_boot_info: CALL read_LINK_word;
				R0=PASS R2;
		
                CALL read_LINK_word;
		
load_memory:	R0=PASS R0;
				IF NE JUMP (PC, test_dm16_zero);

/* After the IDLE completes, the following sequence of instructions will*/
/* be executed: (Remember these are in a loop)                          */
/*      R0=R0-R0, DM(I4,M5)=R9, PM(I12,M13)=R11                         */
/* This instructions sets the EQ flag to terminate the loop, writes 	*/
/* the original value to SYSCON, and writes a *new* instruction over	*/
/* itself.  The new instruction is:                                     */
/*      PM(0, I8)=PX;                                                   */
/* This instruction resets the DMA6 vector to whatever it should be.	*/
/* The loop will terminate, because of the previous set EQ.  Instr		*/
/* flow will continue with 0x40005, just like nothing happened!			*/

final_init:	R9=0xb16b0000;          /* Load instruction PM(0,I8)=PX; 	*/

			R11=BSET R11 BY 9;      /* Set IMDW to 1 for inst write  	*/
			DM(SYSCON)=R11;         /* Setup for instruction write     	*/
			
			R1=0x040000;            /* Point to destination          	*/
			R2=0x100;               /* Load length of last init      	*/
			
			DM(IM8)=M14;            /* Set to increment internal ptr 	*/
			
			I4=0x040004;            /* Point to 0x040004 for patch   	*/
			I8=0x040048;            /* Point to DMA8 vector to patch 	*/
			
			R4=1;
			R4 = pass R4, R11=R12;	/* Clear AZ, hold initial SYSCON 	*/

			DO ___lib_RSTI UNTIL EQ;/* Setup dummy loop              	*/
			FLUSH CACHE;			/* Flush cache for kernel overwrite	*/
						
			R0=0x40004;             /* Clean off old top-of-loop and	*/
			PCSTK=R0;				/* replace w/ new top-of-loop value */

			R0=0x0;
			
			DM(II8)=R1;             /* Setup DMA to load over ldr   	*/
			DM(C8)=R2;				/* Load internal count				*/
			DM(CP8)=R0;				/* Set CP to 0						*/			
						
			DM(LCTL1)=R8;			/* Start DMA transfer				*/
			
			bit clr lirptl 0x10;	/* clear pending LP4 interrupt	 	*/
			bit set lirptl 0x100000;/* enable LP4 interrupt 			*/
			bit set imask 0x4000;	/* enable LPISUM interrupt			*/
			
			JUMP 0x40004 (DB);      /* Jump to start                	*/       
			bit set mode1 0x1800;   /* allow global/nested interrupts  	*/
			jump(pc,-1);			/* wait for interrupt				*/
			{idle;					/* After IDLE, patch then start		*/}
 		

test_dm16_zero:	R0=R0-1;
				IF EQ JUMP (PC, dm16_zero);  
				R0=R0-1;
				IF NE JUMP (PC, test_dm40_zero);

dm32_zero:
dm16_zero:		R0=R0-R0, I0=R3;
				LCNTR=R2, DO dm16_zero_loop UNTIL LCE;	
dm16_zero_loop:		DM(I0,M6)=R0;

				JUMP read_boot_info;

test_dm40_zero:	R0=R0-1;
				IF NE JUMP (PC, test_dm16_init);

dm40_zero:		PX1=0;	
				PX2=0;
				R0=R0-R0, I0=R3;
				LCNTR=R2, DO dm40_zero_loop UNTIL LCE;
dm40_zero_loop:		DM(I0,M6)=PX;

				JUMP read_boot_info;

test_dm16_init:	R0=R0-1;
				IF NE JUMP (PC, test_dm32_init);

dm16_init:		I0=R3;
				LCNTR=R2, DO dm16_init_loop UNTIL LCE;
					CALL read_LINK_word (DB);
                    NOP;               /* NOP's is required by */
                    NOP;               /* loop restriction     */

dm16_init_loop:	DM(I0,M6)=R3;

				JUMP read_boot_info;

test_dm32_init:	R0=R0-1;
				IF NE JUMP (PC, test_dm40_init);

dm32_init:		I0=R3;
				LCNTR=R2, DO dm32_init_loop UNTIL LCE;
					CALL read_LINK_word (DB);
					NOP;	
					NOP;
			
dm32_init_loop:	DM(I0,M6)=R3;

				JUMP read_boot_info;

test_dm40_init:	R0=R0-1;
				IF NE JUMP (PC, test_pm16_zero);

dm40_init:		I0=R3;
				LCNTR=R2, DO dm40_init_loop UNTIL LCE;
					CALL read_LINK_word (DB);
					NOP;
					NOP;

dm40_init_loop:	DM(I0,M6)=PX;
				JUMP read_boot_info;

test_pm16_zero:	R0=R0-1;
				IF EQ JUMP (PC, dm16_zero);
				R0=R0-1;
				IF EQ JUMP (PC, dm32_zero);

test_pm40_zero:	R0=R0-1;
				IF EQ JUMP (PC, dm40_zero);
				R0=R0-1;
				IF NE JUMP (PC, test_pm16_init);

pm48_zero:		PX1=0;
				PX2=0;
				R0=R0-R0, I8=R3;
				LCNTR=R2, DO pm40_zero_loop UNTIL LCE;	
pm40_zero_loop:		PM(I8,M14)=PX;

				JUMP read_boot_info;

test_pm16_init:	R0=R0-1;
				IF EQ JUMP (PC, dm16_init);

test_pm32_init:	R0=R0-1;
				IF EQ JUMP (PC, dm32_init);

test_pm40_init:	R0=R0-1;
				IF EQ JUMP (PC, dm40_init);

test_pm48_init:	R0=R0-1;
				IF NE JUMP test_dm64_zero;

pm48_init:		I8=R3;
				LCNTR=R2, DO pm48_init_loop UNTIL LCE;
					CALL read_LINK_word (DB);
					NOP;
					NOP;

pm48_init_loop:	PM(I8,M14)=PX;
				JUMP read_boot_info;
				
/*************************************************************/				
				
test_dm64_zero:	R0=R0-1;
				IF NE JUMP (PC, test_dm64_init);

dm64_zero:		PX1=0;	
				PX2=0;
				R0=R0-R0, I0=R3;
				LCNTR=R2, DO dm64_zero_loop UNTIL LCE;
dm64_zero_loop:		DM(I0,M6)=PX (lw);

				JUMP read_boot_info;
				

test_dm64_init:	R0=R0-1;
				IF NE JUMP (PC, test_pm64_zero);

dm64_init:		I0=R3;
				LCNTR=R2, DO dm64_init_loop UNTIL LCE;
					CALL read_LINK_word;		/* read 32 lsbs of 64-bit word	*/
					R10=PX2;					/* save 32 lsbs of 64-bit word	*/
					CALL read_LINK_word;		/* read 32 msbs of 64-bit word	*/
					PX1=R10;					/* restore 32 lsbs to PX1 		*/
					nop;						/* Call cannot be in last 3 instructions of loop	*/
dm64_init_loop:		DM(I0,M6)=PX (lw);			/* write all 64-bits to longword address location	*/
				
				JUMP read_boot_info;
				

test_pm64_zero:	R0=R0-1;
				IF EQ JUMP (PC, dm64_zero);


test_pm64_init:	R0=R0-1;
				IF EQ JUMP (PC, dm64_init);
				
				

/*************************************************************/

read_LINK_word:	ustat1=dm(LCOM);			/* Wait for LBUF4 to have data	*/
				bit tst ustat1 0x200;
				if not tf jump read_LINK_word;
				
				PX=PM(LBUF4);				/* Read word from LBUF4			*/
				R2=PX1;						/* Copy PX values into DREGS	*/
				RTS (DB);
				R2=LSHIFT R2 by -16;		/* Downshift 16 bits to adjust for the extension of PX to 64-bits	*/
				R2=PASS R2, R3=PX2;			/* Test the length				*/
.ENDSEG;






