/*------------------------------------------------------------------------------- 
Code to transmit with a ADSP-21160 link port(4) to a ADSP-21062. The DSP's I/O 
processor manages the DMA transfer through the link port. The DMA operation transfers
an entire block of data located in DM data memory. In order to transmit to an 
ADSP-21160 using 8 bit data width the corresponding LxDPWID must be set.
-------------------------------------------------------------------------------*/

#include    "def21160.h"

#define N       512			/* lenght of sample data should be a even number */
        
/*------------ DM data --------------*/
.section/dm seg_dmda;
.VAR source[N]; /*data to be transmitted to SHARC*/

/*---- PM interrupt vector code -----*/
.section/pm   seg_rth;
Reserved_1:     rti; nop; nop; nop;
Chip_Reset:     idle; jump start; nop; nop;

/*------ program memory code -------*/
.section/pm seg_pmco;


		/* computes sample data to be transmitted in one DMA transfer block */	  
start:	bit set MODE1 CBUFEN;
		b0=source; b1=b0; i0=source+N/4; i1=source+3*N/4;
		l0=N; l1=N;	m0=1; r0=N/4; r1=-N/4-1;	  
		/* triangle function period=N, amplitude=N/2 */
		lcntr=N/2, do compute until lce;
		r1=r1+1; dm(i0,m0)=r0;
compute:r0=r0-1, dm(i1,m0)=r1;	/* proper transmission can be easily verified 
								using the debug window plot function */	 

setup:	ustat1 = dm(SYSCON);/* Clear Buffer Hang Disable	*/
		bit clr ustat1 BHD; 
		dm(SYSCON) = ustat1;
		
		r0=0;			/*clear link control registers*/
		dm(LAR)=r0;	    
		dm(LCTL1)=r0;

		/* Set up DMA pointers */
		r0=source;
		dm(II8)=r0;		/* Set DMA channel 8 Index Register */
	
		r0=1;
		dm(IM8)=r0;		/* Set DMA channel 8 Modify Register */

		r0=@source;		
		dm(C8)=r0;		/* Set DMA channel 8 count Register */

		r0=0x4000;		
		dm(LAR)=r0;	 	/* LBUF4->LPORT4, all other ports are disabled */
			
		r0=0x12C00;		/* LBUF4 enable port and dma, transmit, 32bit word size, 1/2 clock, 4bit data path */
		dm(LCTL1)=r0;	/* always write LCTLx after LAR */

/* terminate and wait */
wait1: 
jump wait1;
/*-----------------------------------------------------------------------------*/
