/*------------------------------------------------------------------------------- 
Code to transmit with a ADSP-21160 link port(4) to a ADSP-21062. The core directly 
writes N values to the link buffer(LBUF4). The core stops writing words to the 
buffer, if the L4STAT1 bit indicates a full condition. In order to transmit to an 
ADSP-21160 using 8 bit data width the corresponding LxDPWID must be set.
-------------------------------------------------------------------------------*/

#include    "def21160.h"

#define N       1024
        
/*------------ DM data --------------*/
.section/dm seg_dmda;

 
/*---- PM interrupt vector code -----*/
.section/pm   seg_rth;
Reserved_1:     rti; nop; nop; nop;
Chip_Reset:     idle; jump begin; nop; nop;


/*------ program memory code -------*/

.section/pm seg_pmco;

begin:	r0=0;						/*clear link control registers*/
		dm(LAR)=r0;	    
		dm(LCTL1)=r0;

		ustat1 = dm(SYSCON);		/* Clear Buffer Hang Disable	*/
		bit clr ustat1 BHD; 
		dm(SYSCON) = ustat1;
		
		r0=0x4000;					/* LBUF4->LPORT4, all other ports are disabled */
		dm(LAR)=r0;	 

		r0=L4EN | L4CLKD1 | L4TRAN; /*LBUF4 enable port and dma, transmit, 
									32bit word size, 1/2 clock, 4bit data path*/
		dm(LCTL1)=r0;				/*always write LCTLx after LAR*/	
		r1=1;
		r2=N;
		
readstat:
		ustat1=dm(LCOM);			/* read Link Port common Control Register */
		bit tst ustat1 L4STAT1;		
		if tf jump readstat;		/* test until buffer is empty */
		if GT jump readstat (DB);
		dm(LBUF4)=r2;				/* write value to link port buffer */
		r2=r2-r1;
			


/* terminate and wait */
wait1: 
jump wait1;
/*-----------------------------------------------------------------------------*/
