/*------------------------------------------------------------------------------- 
Code to receive with a ADSP-21062 link port(5) from a ADSP-21160. The core directly 
reads N values from the link buffer(LBUF2). The core will hang on the read of 
LBUF4 until data is ready to receive. Received values are stored DM Memory. 
-------------------------------------------------------------------------------*/


#include	"def21060.h"

#define N 1024

/*------------ DM data --------------*/
.section/dm seg_dmda;
.var dest[N];

/*---- PM interrupt vector code -----*/
.section/pm   seg_rth;
Reserved_1:     rti; nop; nop; nop;
Chip_Reset:     idle; jump start; nop; nop;

/*------ program memory code -------*/

.section/pm seg_pmco;

start:  ustat1 = dm(SYSCON);	/* Clear Buffer Hang Disable	*/
		bit clr ustat1 0x800; 
		dm(SYSCON) = ustat1;

		b1=dest;				/*setup circular buffer*/
		l1=N;
		m1=1;

setup:	r0=0x00000000;			
		dm(LCTL)=r0;			/*clear link control registers*/
		dm(LAR)=r0;				

		r0=0x0000000;			/*LCKX2x transfers occurring at CK frequency */
		dm(LCOM)=r0;

		r0=0x3ff7f;				/* 	LBUF2 <==> LPORT5, all others disabled */
		dm(LAR)=r0;

		r9=0x00000100;			/* lbuf2 enable, receive, 32-bit word, 4-bit bus width */
		dm(LCTL)=r9;
		r2=N;
		
readstat:

		ustat1=dm(LCOM);		/* read Link Port common Control Register */
		bit tst ustat1 0x10;	/* test buffer full */
		if not tf jump readstat;/* test until word is received */
		r2=r2-1;				/* count received words */
		if GT jump readstat (DB);
		r1=dm(LBUF2);
		dm(i1,m1)=r1;			/* save received words until count expired */

		/* terminate and wait */
wait1:	idle;
		jump wait1;
/*-----------------------------------------------------------------------------*/
.ENDSEG;

